* ------------------------------------------------------
*  Special Function Registers

       .include "sfr.i"

* ------------------------------------------------------
*  Variable-Definitions

;Oscillation Control Register settings
;Internal RC oscillation for system clock
osc_rc EQU 129
;Crystal oscillation for system clock
osc_xt EQU 130

* ------------------------------------------------------
*  Interupt Vectors

	.include "libraries/intvec.i"

* ------------------------------------------------------
*  Game Data

	.include "data/gamehdr.i"
	.include "data/iconanm.i"
	.include "data/iconplt.i"
	.include "data/icondat.i"

* ------------------------------------------------------
*  Main Program Loop

	.include "libraries/main.i"

* ------------------------------------------------------
*  Jump To User Program

	;jmp begin

* ------------------------------------------------------
*  User Defined Imports


* ------------------------------------------------------
*  User Defined Program

begin:
	;Display data
	mov #$f0, c
	;Display pattern on the LCD
	call matrix
	;Enters HALT mode and waits for an interrupt
	;HALT mode is cancelled and processing continues
	;When a base timer interrupt is generated
	set1 PCON, 0
	
	;The following lines display different patterns in the same manner
	mov #$0f, c
	call matrix
	set1 PCON, 0
	
	mov #$cc, c
	call matrix
	set1 PCON, 0

	mov #$33, c
	call matrix
	set1 PCON, 0

	mov #$55, c
	call matrix
	set1 PCON, 0

	mov #$aa, c
	call matrix
	set1 PCON, 0

	;[M] (mode) Button Check
	ld p3
	;If the [M] button is pressed the application ends
	bn acc, 6, .finish
	;Repeat back to program start
	jmp begin
.finish:
	call finish

;End program
finish:
	jmpf goodbye

;Draws one LCD screen
matrix:
	;Pushes each register onto the stack
	push acc
	push b
	push c
	push XBNK
	
xb0_a:
	;Specifies the display RAM bank address (BANK0)
	mov #$00, XBNK
	mov #$80, b

la1:
	;c: Display data
	ld c
	;2-line display
	call line2
	;Advances 2 lines ahead
	ld b
	add #$10
	st b
	;Repeats until end of bank is reached
	bnz la1
	
xb1_a:
	;Specifies the display RAM bank address (BANK1)
	mov #$01, XBNK
	mov #$80, b

la2:
	;c: Display data
	ld c
	;2-line display
	call line2
	;Advances 2 lines ahead
	ld b
	add #$10
	st b
	;Repeats until end of bank is reached
	bnz la2
	
	;Pops the registers from the stack
	pop XBNK
	pop c
	pop b
	pop acc
	;Matrix end
	ret

;LCD 2-line display	
line2:
	;Pushes each register onto the stack
	push acc
	push b
	push c
	push PSW
	push OCR
	;Specifies the system clock
	mov #osc_rc, OCR
	;Selects data RAM bank 1
	set1 PSW, 1
	;Stores display data in c
	st c
	;Sets the display RAM addresses in r2
	ld b
	st 2

;First line display processing
lp1:
	;Transfers the display data to display RAM
	ld c
	st @r2
	;Advances the address to the next display position
	inc 2
	ld 2
	;If the display position is not at the right end of the first line...
	and #$0f
	xor #$06
	;... Repeat
	bnz lp1

	;Inverts the bit pattern in the c register	
	ld c
	xor #$ff
	st c

;Second line display processing
lp2:
	;Transfers the display data to the display RAM
	ld c
	st @r2
	;Advances the address to the next display position
	inc 2
	ld 2
	;If the display position is not at the right end of the second line...
	and #$0f
	xor #$0c
	;... Repeat
	bnz lp2

	;Pops registers off the stack	
	pop OCR
	pop PSW
	pop c
	pop b
	pop acc
	;Line2 end
	ret

* ------------------------------------------------------
*  End Program

	.include "libraries/endprog.i"

