;Serial Communications Sample 1 (Data Transmission)
;Sends simple data through the serial communications port on a regular cycle

* ------------------------------------------------------
*  Special Function Registers

       .include "sfr.i"

* ------------------------------------------------------
*  Variable-Definitions

;Oscillation Control Register settings
;Internal RC oscillation for system clock
osc_rc EQU 77
;Crystal oscillation for system clock
osc_xt EQU 239

;Low battery detection flag (RAM bank 0)
LowBattChk EQU 110

;Counter
counter = $30

* ------------------------------------------------------
*  Interupt Vectors

	.include "libraries/c_intvec.i"

* ------------------------------------------------------
*  Game Data

	.include "data/gamehdr.i"
	.include "data/iconanm.i"
	.include "data/iconplt.i"
	.include "data/icondat.i"

* ------------------------------------------------------
*  Main Program Loop

	.include "libraries/main.i"

* ------------------------------------------------------
*  Jump To User Program

	;jmp begin

* ------------------------------------------------------
*  User Defined Imports


* ------------------------------------------------------
*  User Defined Program

begin:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 1
	set1 PSW, 1
	;Turns off the low battery automatic detection function
	call BattChkOff

cwait:
	;Serial communications initialization
	call SioInit
	;Starts if VM is connected
	bz startprog

	;[M] button check
	ld P3
	;If the [M] button is pressed, the application ends
	bn acc, 6, .finish
	;Waits until VM is connected
	jmp cwait
.finish:
	call finish

startprog:
	;Waits in HALT mode until next interrupt (0.5 seconds)
	set1 PCON, 0
	;Resets the counter value to "0"
	mov #0, counter

loop0:
	;Loads the counter value
	ld counter
	;Sends one byte
	call SioSend1
	;Waits in HALT mode until next interrupt (0.5 seconds)
	set1 PCON, 0
	;[M] (mode) Button Check
	ld P3
	;If the [M] button is pressed, the application ends
	bn acc, 6, .finish
	;Repeat
	jmp loop0
.finish:
	call finish

;End program
finish:
	;Serial communications end processing
	call SioEnd
	;Turns on the low battery automatic detection function
	call BattChkOn
	;Pops the PSW value off of the stack
	pop PSW
	jmpf goodbye

;Serial Communications Initialization
;Outputs
;	acc = 0 : Normal end
;	acc = 0ffh: VM not connected

;Serial communications initialization
;This sample assumes that the system clock is in crystal mode.
SioInit:
	;VM Connection Check
	;Checks the connection status
	ld P7
	;Checks P70, P72, P73
	and #%00001101
	;P70 = 0, P72 = 0, P73 = 1
	sub #%00001000
	;To next2 if connected
	bz next2
	;If not connected, abnormal end with acc = 0ffh
	mov #$ff, acc
	;SioInit end
	ret

;Serial communications initialization
next2:
	;Specifies output as 'LSB first'
	mov #0, SCON0
	;Specifies input as 'LSB first'
	mov #0, SCON1
	;Sets the transfer rate
	mov #$dd, SBR
	;Clears the P10 latch (P10/S00)
	clr1 P1, 0
	;Clears the P12 latch (P12/SCK0)
	clr1 P1, 2
	;Clears the P13 latch (P13/S01)
	clr1 P1, 3
	;Sets the pin functions
	mov #%00000101, P1FCR
	;Sets the pin functions
	mov #%00000101, P1DDR
	;Clears the transfer buffer
	mov #0, SBUF0
	;Clears the transfer buffer
	mov #0, SBUF1
	;SioInit end
	ret

;Serial communications end
;Serial communications end processing
SioEnd:
	;SCON0 = 0
	mov #0, SCON0
	;SCON0 = 1
	mov #0, SCON1
	;P1FCR = 0bfh
	mov #$bf, P1FCR
	;P1DDR = 0a4h
	mov #$a4, P1DDR
	;SioEnd end
	ret

;Sending 1 byte from a serial port
;	Inputs: acc: transmission data
;Sending 1 byte
SioSend1:
	;Pushes the transmission data onto the stack
	push acc
sslp1:
	;Waits, if the previous data is still being sent
	ld SCON0
	bp acc, 3, sslp1
	;Pops the transmission data off of the stack
	pop acc
	;Sets the data to be transferred
	st SBUF0
	;Starts sending
	set1 SCON0, 3
	;SioSend1 end
	ret

;Low battery automatic detection function ON
BattChkOn:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 0
	clr1 PSW, 1
	;Detects low battery (0)
	mov #0, acc
	;Low battery automatic detection flag (RAM bank 0)
	st LowBattChk
	;Pops the PSW value off of the stack
	pop PSW
	;BattChkOn end
	ret

;Low battery automatic detection function OFF
BattChkOff:
	;Pushes the PSW value onto the stack
	push PSW
	;Selects data RAM bank 0
	clr1 PSW, 1
	;Does not detect low battery (0ffh)
	mov #$ff, acc
	;Low battery automatic detection flag (RAM bank 0)
	st LowBattChk
	;Pops the PSW value off of the stack
	pop PSW
	;BattChkOff end
	ret

;Base Timer Interrupt Handler
int_BaseTimer:
	;Pushes the PSW value onto the stack
	push PSW
	push acc
	;Selects data RAM bank 1
	set1 PSW, 1
	;Increments the counter
	inc counter
	;If the counter value is:
	ld counter
	;not 100, then next1
	bne #100, next1
	;100, then reset to '0'
	mov #0, counter

next1:
	pop acc
	;Pops the PSW value off of the stack
	pop PSW
	;Clears the base timer interrupt source
	clr1 BTCR, 1
	;User interrupt processing end
	ret

* ------------------------------------------------------
*  End Program

	.include "libraries/endprog.i"

